--------------------------------------------------------------------------------
-- Demo Derby Vehicle Randomiser
-- By Dagalufh & Scepheo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Scenario config
--------------------------------------------------------------------------------

local vehicleNames = 
{
	"scenario_player0",
	"opponent1",
	"opponent2",
	"opponent3",
}

--------------------------------------------------------------------------------
-- Scenario code
--------------------------------------------------------------------------------

local M = {}

local function getRandomKey(items)
	-- Count the number of items in the table
	local itemCount = 0
	
	for key, value in pairs(items) do
		itemCount = itemCount + 1
	end
	
	-- Select a random number n
	local selectedIndex = math.random(1, itemCount)
	
	-- Get the nth node
	local index = 1
	local selectedKey = ""
	
	for key, value in pairs(items) do
		if index == selectedIndex then
			selectedKey = key
			break
		end
		
		index = index + 1
	end
	
	return selectedKey
end

local function getRandomValue(items)
	return items[getRandomKey(items)]
end

local function onPreStart(scenario)

	-- Get the list of vehicles
	local vehicleList = vehicles.getModelList().models
	
	-- Get a new random seed
	math.randomseed(os.time())
	
	-- Replace each vehicle with a random one
	for i, carName in pairs(vehicleNames) do
		local vehicle = {}
		local valid = false
		
		while not valid do
			vehicle = getRandomValue(vehicleList)
			valid = vehicle.Type == "Car" or vehicle.Type == "Truck"
		end
		
		local jbeam = vehicle.key
		local config = vehicle.Configuration
		
		local configPath = ''
		if (vehicle.ConfigName) then
			configPath = 'vehicles/' .. jbeam .. '/' .. vehicle.ConfigName .. '.pc'
		end
		
		local color
		
		if vehicle.colors == nil then
			color = "0 0 0 1"
		else
			color = getRandomValue(vehicle.colors)
		end
		
		scenario.demoNames[i - 1] = vehicle.key
		
		TorqueScript.eval(
		[[
			]] .. carName .. [[.JBeam = "]] .. jbeam .. [[";
			]] .. carName .. [[.partConfig = "]] .. configPath .. [[";
			]] .. carName .. [[.color = "]] .. color .. [[";
			]] .. carName .. [[.requestReload();
		]])
	end
end

local function onScenarioChange(scenario)
	if scenario and scenario.state == 'pre-start' then
		onPreStart(scenario)
	end
end

local function onPhysicsEngineEvent(eventType)
	-- Freeze whenever a vehicle is spawned or despawned
	scenetree.scenario_player0:queueLuaCommand('scenario.freeze(1)')
end

M.onScenarioChange = onScenarioChange
M.onPhysicsEngineEvent = onPhysicsEngineEvent

return M